//
// C++ Implementation: wwCanvasView
//
// Description:
//
//
// Author: Witold Wysota <wysota@wysota.eu.org>, (C) 2005-2006
//
//
//
#include "wwcanvasview.h"
#include <qwmatrix.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qaction.h>

wwCanvasView::wwCanvasView( QWidget * parent, const char * name, WFlags f )
        : QCanvasView( parent, name, f ) {
    m_dragged = 0;
    m_dragsEnabled = false;
}

wwCanvasView::wwCanvasView( QCanvas * canvas, QWidget * parent, const char * name, WFlags f )
        : QCanvasView( canvas, parent, name, f ) {
    m_dragged = 0;
    m_dragsEnabled = false;
}

wwCanvasView::~wwCanvasView() {}

void wwCanvasView::contentsMousePressEvent( QMouseEvent * e ) {
    if(e->button()!=LeftButton || !canvas())
        return QCanvasView::contentsMousePressEvent(e);
    QWMatrix wm = inverseWorldMatrix();
    QPoint pt = wm.map(e->pos());
    QCanvasItemList l = canvas()->collisions(pt);
    if(l.empty()) {
        emit clicked(0);
        return QCanvasView::contentsMousePressEvent(e);
    }
    if(!m_dragsEnabled) {
        emit clicked(l.first());
        return QCanvasView::contentsMousePressEvent(e);
    }
    QCanvasItem *clicked_item = l.first();
    m_dragged = clicked_item;
    m_dragorig = pt;
    emit clicked(clicked_item);
}


void wwCanvasView::contentsMouseReleaseEvent( QMouseEvent * e ) {
    if(!m_dragsEnabled)
        return QCanvasView::contentsMouseReleaseEvent(e);
    m_dragged = 0;
    m_dragorig=QPoint();
}

void wwCanvasView::contentsMouseMoveEvent( QMouseEvent * e ) {
    if(!m_dragsEnabled)
        return QCanvasView::contentsMouseMoveEvent(e);
    if(!m_dragged)
        return;
    QWMatrix wm = inverseWorldMatrix();
    QPoint pt = wm.map(e->pos());
    emit dragged(m_dragged, pt);
    m_dragged->moveBy(pt.x()-m_dragorig.x(), pt.y()-m_dragorig.y());
    m_dragorig = pt;
    canvas()->update();
}


void wwCanvasView::contentsContextMenuEvent( QContextMenuEvent * e ) {
    if(m_actionlist.isEmpty())
        return QCanvasView::contentsContextMenuEvent(e);
    QPopupMenu menu;
    for(QPtrList<QAction>::iterator it = m_actionlist.begin(); it!=m_actionlist.end();++it) {
        (*it)->addTo(&menu);
    }
    menu.exec(QCursor::pos());
}


/**
 * @brief Scales the canvas in X direction
 * @param sx scale factor
 */
void wwCanvasView::setScaleX( const double sx ) {
    if(!canvas())
        return;
    QWMatrix wm = worldMatrix();
    wm.scale(sx, worldMatrix().m22());
    setWorldMatrix(wm);
    canvas()->update();
}

/**
 *  @brief Scales the canvas in Y direction
 *  @param sy scale factor
 */
void wwCanvasView::setScaleY( const double sy ) {
    if(!canvas())
        return;
    QWMatrix wm = worldMatrix();
    wm.scale(worldMatrix().m11(), sy);
    setWorldMatrix(wm);
    canvas()->update();
}

/**
 *  @brief Returns the scale of Y axis
 *  @return scale of Y axis
 */
double wwCanvasView::scaleY( ) const {
    return worldMatrix().m11();
}

/**
 *  @brief Returns the scale of X axis
 *  @return scale of X axis
 */
double wwCanvasView::scaleX( ) const {
    return worldMatrix().m22();
}

/**
 * @brief Sets the scale of both X and Y axis
 * @param sx scale factor for X axis
 * @param sy scale factor for Y axis
 */
void wwCanvasView::setScale( double sx, double sy ) {
    if(!canvas())
        return;
    if(sy==0)
        sy=sx;
    QWMatrix wm = worldMatrix();
    wm.scale(sx,sy);
    setWorldMatrix(wm);
    canvas()->update();
}

/**
 *  @brief Doubles the scale of the canvas
 */
void wwCanvasView::Double( ) {
    setScale(2, 2);
}

/**
 *  @brief Halves the scale of the canvas
 */
void wwCanvasView::Halve( ) {
    setScale(0.5, 0.5);
}

/**
 * @brief Adds action to the list of actions displayed as the context menu
 * @param a action to add
 */
void wwCanvasView::addAction( QAction * a ) {
    m_actionlist.append(a);
}

/**
 * @brief Removes an action from the list of actions displayed as the context menu
 * @param a action to remove
 */
void wwCanvasView::removeAction( QAction * a ) {
    m_actionlist.remove(a);
}

/**
 * @brief Enables or disables dragging of objects
 * @param yes boolean value stating whether dragging should be enabled
 */
void wwCanvasView::setDragsEnabled( bool yes ) {
    m_dragsEnabled = yes;
}

/**
 * @brief Returns information if dragging is enabled
 * @return true if dragging is enabled, false otherwise
 */
bool wwCanvasView::dragsEnabled( ) const {
    return m_dragsEnabled;
}

